/*
 * TestServlet.java
 */

package ejb30.inherit.servlet;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.*;
import java.util.*;
import java.io.*;

import ejb30.inherit.session.*;
import ejb30.inherit.entity.*;
import javax.ejb.*;

public class TestServlet extends HttpServlet{
    
    private static final String CREATE_ORDER="createOrder";
    
    @EJB 
    private SessionLocal sessionLocal;
    
    private InitialContext ctx;
    
    /**
     * Creates a new instance of TestServlet
     */
    public TestServlet() {
        try{
            ctx = new InitialContext();
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
 
 
    protected void doGet( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException {
        processAction(request, response);
    }
    
    protected void doPost( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException {
        processAction(request, response);
    }
    
    private void processAction(HttpServletRequest request, HttpServletResponse response) throws IOException{
    
        System.out.println("processing test driver request ... ");
        
        String action = request.getParameter("action");
        System.out.println("Action:"+action);

        response.setContentType("text/html");
        ServletOutputStream out = response.getOutputStream();
                
        if(action == null || "".equals(action)){
            //do nothing, just redirect to index.html
            response.sendRedirect("index.html");
        }else if("createAccount".equalsIgnoreCase(action)) {
            String accountType = request.getParameter("accountType");
            System.out.println("AccountType :"+accountType);
            
            
            out.println(header);
            
            out.println(renderAccountForm());
            
            if("checking".equalsIgnoreCase(accountType)){
                out.println(renderCheckingForm());
            }else if("savings".equalsIgnoreCase(accountType)){
                out.println(renderSavingsForm());
            }else if("creditcard".equalsIgnoreCase(accountType)){
                out.println(renderCreditCardForm());
            }else if("brokerage".equalsIgnoreCase(accountType)){
                out.println(renderBrokerageForm());
            }else if("margin".equalsIgnoreCase(accountType)){
                out.println(renderMarginForm());
            }else{
                
            }
            
            out.println(renderSubmit());
            out.println(footer);
            
        }else if ("persistAccount".equalsIgnoreCase(action)){

            String accountType = request.getParameter("accountType");
            System.out.println("AccountType :"+accountType);
            
            String acctNum = request.getParameter("acctNum");
            String name = request.getParameter("acctName");
            String balance = request.getParameter("balance");
            String description = request.getParameter("description");
            String status = request.getParameter("status");

            Account account=null; 
            
            if("checking".equalsIgnoreCase(accountType)){
                String overdraft = request.getParameter("overdraft");
                
                account = new CheckingAccount();
                ((CheckingAccount)account).setIsOverDraftAllowed("yes".equalsIgnoreCase(overdraft));
                
            }else if("savings".equalsIgnoreCase(accountType)){
                
                float fRate=0;
                try{
                    fRate=Float.valueOf(request.getParameter("savingsRate"));
                }catch(Exception ex){
                    ex.printStackTrace();
                }
                
                account = new SavingsAccount();
                ((SavingsAccount)account).setSavingsRate(fRate);
                
            }else if("creditcard".equalsIgnoreCase(accountType)){
                String ccNum = request.getParameter("ccnum");
                String bank = request.getParameter("bank");
                
                account = new CreditCardAccount();
                ((CreditCardAccount)account).setCreditCardNumber(ccNum);
                ((CreditCardAccount)account).setIssuingBank(bank);
                
                
            }else if("brokerage".equalsIgnoreCase(accountType)){
                float tradeFees=0;
                try{
                    tradeFees=Float.valueOf(request.getParameter("tradeFees"));
                }catch(Exception ex){
                    ex.printStackTrace();
                }
                
                account = new BrokerageAccount();
                ((BrokerageAccount)account).setTradeFees(tradeFees);
                
            }else if("margin".equalsIgnoreCase(accountType)){
                
                float tradeFees=0;
                try{
                    tradeFees=Float.valueOf(request.getParameter("tradeFees"));
                }catch(Exception ex){
                    ex.printStackTrace();
                }
                
                float maxLoan=0;
                try{
                    maxLoan=Float.valueOf(request.getParameter("maxLoan"));
                }catch(Exception ex){
                    ex.printStackTrace();
                }
                
                account = new MarginAccount();
                ((MarginAccount)account).setMaxLoanAllowed(maxLoan);
                ((MarginAccount)account).setTradeFees(tradeFees);
                
            }else{
                
            }
            
            float fBalance=0;
            try{
                fBalance=Float.valueOf(request.getParameter("balance"));
            }catch(Exception ex){
                ex.printStackTrace();
            }
            
            String szStatus = request.getParameter("status");
            
            
            account.setAcctNum(request.getParameter("acctNum"));
            account.setBalance(fBalance);
            account.setDescription(request.getParameter("description"));
            account.setName(name);
            
            if("open".equalsIgnoreCase(szStatus)){
	       account.setStatus(Account.Status.OPEN);
            }else{
	      account.setStatus(Account.Status.CLOSED);
            }
            
            sessionLocal.persistAccountL(account);
            
            out.println(header);
            out.println("<tr><td colspan=\"2\" align=\"center\">Account information was successfully saved.</td></tr>");
            out.println(footer);
            
        }else if ("findAccount".equalsIgnoreCase(action)){
            
            String acctNum = request.getParameter("acctNum");
            Account account = sessionLocal.findAccountL(acctNum);

            out.println(header);
            
            if(account!=null){
                out.println(new StringBuffer("<tr><td colspan =\"2\"><b>Found ").append(account.getClass().getSimpleName()).append(" with account number:").append(acctNum).append("</b></td></tr>").toString());
                out.println("<tr><td colspan=\"2\">&nbsp;</td></tr>");
                out.println("<tr><td colspan=\"2\"><b>Account Details:</b></td></tr>");
                out.println(renderAccountDetails(account));
            }else{
                out.println("<tr><td colspan =\"2\">No account with account number:"+acctNum+ "</td></tr>");
            }
            out.println(footer);
            
        }
       
    }
    
    private String renderAccountForm(){
        
        StringBuffer sb = new StringBuffer();
        
        sb.append("<tr><td>Account Number</td><td><input type=\"text\" name=\"acctNum\"></td></tr>");
        sb.append("<tr><td>Name</td><td><input type=\"text\" name=\"acctName\"></td></tr>");
        sb.append("<tr><td>Status</td><td><select name=\"status\"><option value=\"open\" selected=\"true\">Open</option><option value=\"closed\">Closed</option></select></td></tr>");
        sb.append("<tr><td>Description</td><td><input type=\"text\" name=\"description\"></td></tr>");
        sb.append("<tr><td>Balance</td><td><input type=\"text\" name=\"balance\"></td></tr>");
        
        return sb.toString();
        
        
    }
    
    private String renderCheckingForm(){
        
        StringBuffer sb = new StringBuffer();
        sb.append("<tr><td>Overdraft allowed</td><td><select name=\"overdraft\"><option value=\"yes\" selected=\"true\">Yes</option><option value=\"no\">No</option></select></td></tr>");
        sb.append("<tr><td colspan=\"2\" align=\"center\"> <input type=\"hidden\" name=\"accountType\" value=\"checking\"></td></tr>");
        return sb.toString();
    
    }
    
    private String renderSavingsForm(){
        
        StringBuffer sb = new StringBuffer();
        sb.append("<tr><td>Savings Rate</td><td><input type=\"text\" name=\"savingsRate\"></td></tr>");
        sb.append("<tr><td colspan=\"2\" align=\"center\"> <input type=\"hidden\" name=\"accountType\" value=\"savings\"></td></tr>");
        return sb.toString();
        
    }
    
    private String renderCreditCardForm(){
        
        StringBuffer sb = new StringBuffer();
        sb.append("<tr><td>Credit Card Number</td><td><input type=\"text\" name=\"ccnum\" maxlength=\"16\"></td></tr>");
        sb.append("<tr><td>Issuing Bank</td><td><input type=\"text\" name=\"bank\"></td></tr>");
        sb.append("<tr><td colspan=\"2\" align=\"center\"> <input type=\"hidden\" name=\"accountType\" value=\"creditcard\"></td></tr>");
        return sb.toString();
        
    }
    
    private String renderBrokerageForm(){
        
        StringBuffer sb = new StringBuffer();
        sb.append("<tr><td>Trading Fees</td><td><input type=\"text\" name=\"tradeFees\"></td></tr>");
        sb.append("<tr><td colspan=\"2\" align=\"center\"> <input type=\"hidden\" name=\"accountType\" value=\"brokerage\"></td></tr>");
        return sb.toString();
        
    }
    
    private String renderMarginForm(){
        
        StringBuffer sb = new StringBuffer();
        sb.append("<tr><td>Trading Fees</td><td><input type=\"text\" name=\"tradeFees\"></td></tr>");
        sb.append("<tr><td>Max loan allowed </td><td><input type=\"text\" name=\"maxLoan\"></td></tr>");
        sb.append("<tr><td colspan=\"2\" align=\"center\"> <input type=\"hidden\" name=\"accountType\" value=\"margin\"></td></tr>");
        return sb.toString();
        
    }
    
    private String renderSubmit(){
        StringBuffer sb = new StringBuffer();
        sb.append("<tr><td colspan=\"2\"> <input type=\"submit\" name=\"submit\" value=\"Submit\"></td></tr>");
        sb.append("<tr><td colspan=\"2\" align=\"center\"> <input type=\"hidden\" name=\"action\" value=\"persistAccount\"></td></tr>");
        return sb.toString();
    }
    
    private String renderAccountDetails(Account account){
        
        StringBuffer sb = new StringBuffer();
        
        sb.append("<tr><td>Account Number</td><td>").append(account.getAcctNum()).append("</td></tr>");
        sb.append("<tr><td>Name</td><td>").append(account.getName()).append("</td></tr>");

        sb.append("<tr><td>Status</td><td>").append(account.getStatus()==Account.Status.OPEN?"Open":"Closed").append("</td></tr>");
        sb.append("<tr><td>Description</td><td>").append(account.getDescription()).append("</td></tr>");
        sb.append("<tr><td>Balance</td><td>").append(account.getBalance()).append("</td></tr>");
        
        if(account instanceof CheckingAccount){
            
            sb.append("<tr><td>Overdraft allowed</td><td>").append(((CheckingAccount)account).isIsOverDraftAllowed()==true?"Yes":"No").append("</td></tr>");
        
        }else if(account instanceof SavingsAccount){
            
            sb.append("<tr><td>Savings Rate</td><td>").append(((SavingsAccount)account).getSavingsRate()).append("</td></tr>");
            
        }else if(account instanceof CreditCardAccount){
            sb.append("<tr><td>Credit Card Number</td><td>").append(((CreditCardAccount)account).getCreditCardNumber()).append("</td></tr>");
            sb.append("<tr><td>Issuing Bank</td><td>").append(((CreditCardAccount)account).getIssuingBank()).append("</td></tr>");
        
        }else if (account instanceof MarginAccount){
            
            sb.append("<tr><td>Trading Fees</td><td>").append(((MarginAccount)account).getTradeFees()).append("</td></tr>");
            sb.append("<tr><td>Max loan allowed </td><td>").append(((MarginAccount)account).getMaxLoanAllowed()).append("</td></tr>");
        
        }else if (account instanceof BrokerageAccount){
            sb.append("<tr><td>Trading Fees</td><td>").append(((BrokerageAccount)account).getTradeFees()).append("</td></tr>");
        }
        
        return sb.toString();
        
    }
    
    private static final String header="<html><head><title>Java Persistence: Inheritance Test</title></head><body><form name=\"submitForm\" action=\"/ejb30-inherit/TestServlet\" method=\"post\"><table cellspacing=\"3\" cellpadding=\"3\" align=\"center\" width=\"50%\" valign=\"center\"><th>Java Persistence: Inheritance Test</th><tr><td>&nbsp</td></tr>";
    private static final String footer="<tr><td colspan=\"2\" align=\"center\"><a href=\"/ejb30-inherit/index.html\">Home</a></td></tr></table></form></body></html>";
    
}
